/*
 * LCD_UART.c
 *
 * LCD Display
 *
 *
 *
 * Created: 2012/01/07 22:30:37
 *  Author: Satoshi
 */ 

#include "cpu_def.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "Lcd.h"
#include "UART.h"


int main(void)
{
	int8_t  x, y, n1, n2;
	uint8_t esc;
	char    d;

	DDRB  = 0b00011111;		// 7:   6:   5:    4:E   3-0:data 
	DDRD  = 0b01100000;		// 7:   6:RS 5:LED 4:    3:    2:    1:    0: 
	PORTB = 0;
	PORTD = 0;	

    sei();								// S荞݋
	
	wdt_enable( WDTO_500MS );

	LcdTextInitialize();

	UartInitialize( 9800 );				// Initalize of UART
    UartSendString( "\n\rOK." );

	x = y = 0;
	LcdTextClear( );
	LcdTextLED ( LCD_DISPLAY_ON );
	
	LcdTextPutstr( "Ready.");
	
	LcdTextLocate(2, 2);

	esc = 0;
	n1 = n2 = 0;
    for(;;) {
        d = UartGetReceivedData();
		if (d != 0) {
			if (esc == 0) {
				switch( d ) {
				case '\r':					// CR
					x = 0;
					LcdTextLocate(x, y);
					break;
				case '\n':					// LF
					y+=1;
					if (y > LCD_MAX_Y) y = LCD_MAX_Y;
					LcdTextLocate(x, y);
					break;
				case '\b':					// BS
					x-= 1;
					if (x < 0) x = 0;
					LcdTextLocate( x, y );
					LcdTextPutch( ' ' );
					LcdTextLocate( x, y );
					break;
				case '\f':					// FF (Clear)
					x = y = 0;
					LcdTextClear( );
					break;
				case (char)0x11:			// LED On
					LcdTextLED( LCD_LED_ON );
					break;
				case (char)0x12:			// LED Off
					LcdTextLED( LCD_LED_OFF );
					break;
				case (char)0x1b:			// ESC
					esc = 1;
					break;
				default:
					if (d >= 0x20) {
						LcdTextPutch( d );
						x += 1;
						if (x > LCD_MAX_X) {
							x  = 0;
							y += 1;
							if (y > LCD_MAX_Y) y = LCD_MAX_Y;
						}
					}
					break;
				}
			}
			else if (esc == 1) {
				if (d == '[') esc = 2;
				else          esc = 0;
			}
			else if (esc == 2) {
				if (d == '>') esc = 10;
				else {
					n1 = (uint8_t)d - (uint8_t)'0';
					esc = 3;
				}
			}
			else if (esc == 3) {
				if (d >= '0' && d <= '9') {
					n1 = n1 * 10 + 	(uint8_t)d - (uint8_t)'0';
				}				
				else if (d == ';') {					// Locate
					esc = 4;
				}
				else {
					if (d == 'A') {					// Cursor n Up
						y = y - n1;
						if (y < 0) y = 0;
					}
					if (d == 'B' ) {				// Cursor n Down
						y = y + n1;
						if (y > LCD_MAX_Y) y = LCD_MAX_Y;
					}
					if (d == 'C' ) {				// Cursor n Left
						x = x - n1;
						if (x < 0) x = 0;
					}
					if (d == 'D' ) {				// Cursor n Right
						x = x + n1;
						if (x > LCD_MAX_X) x = LCD_MAX_X;
					}
					LcdTextLocate(x, y);
					esc = 0;
				}
			}
			else if (esc == 4) {				// Locate
				n2 = (uint8_t)d - (uint8_t)'0';
				esc = 5;
			}
			else if (esc == 5) {				// Locate
				if (d >= '0' && d <= '9') {
					n2 = n2 * 10 + (uint8_t)d - (uint8_t)'0';
				}
				else if (d == 'H') {
					n1 -= 1;
					n2 -= 1;
					if (n1 < 0) n1 = 0;
					if (n2 < 0) n2 = 0;
					if (n1 > LCD_MAX_Y) n1 = LCD_MAX_Y;
					if (n2 > LCD_MAX_X) n2 = LCD_MAX_X;
					x = n2;
					y = n1;
					LcdTextLocate(x, y);
					esc = 0;
				}
				else esc = 0;
			}
			else if (esc == 10) {
				switch( d ) {
				case '1':
					esc = 11;
					break;
				case '3':
					esc = 13;
					break;
				case '5':
					esc = 15;
					break;
				case '6':
					esc = 16;
					break;
				default:
					esc = 0;
				
				}
			}
			else if (esc == 11) {		// Cursor Brink
				if (d == 'l')       LcdTextMode( LCD_BRINK_ON );
				else if (d == 'h' ) LcdTextMode( LCD_BRINK_OFF );
				esc = 0;
			}
			else if (esc == 13) {		// Shift Left/Right
				if (d == 'l')       LcdTextCursor( LCD_CURSOR_RIGHT_D );
				else if (d == 'h' ) LcdTextCursor( LCD_CURSOR_LEFT_D );
				esc = 0;
			}
			else if (esc == 15) {		// Cursor shift
				if (d == 'l')       LcdTextMode( LCD_CURSOR_ON );
				else if (d == 'h' ) LcdTextMode( LCD_CURSOR_OFF );
				esc = 0;
			}
			else if (esc == 16) {		// Cursor shift Left/Right
				if (d == 'l')       LcdTextCursor( LCD_CURSOR_RIGHT );
				else if (d == 'h' ) LcdTextCursor( LCD_CURSOR_LEFT );
				esc = 0;
			}
			else esc = 0;
		}
		wdt_reset();
    }
}