/*
 * Uart.c
 *
 * UART Interface
 * ATtiny2313   2008.12.02
 *
 * Created: 2012/01/07 22:35:00
 *  Author: Satoshi
 */

#include "cpu_def.h"

#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"

#define RBUF_MAX	32					// Size of Recive buffer
#define SBUF_MAX	4					// Size of Send buffer

volatile unsigned char rbuf[RBUF_MAX];	// USART Recive buffer (ring buffer)
volatile uint8_t rwpnt;					// Recive buffer write pointer
volatile uint8_t rrpnt;					// Recive buffer read  pointer
volatile unsigned char sbuf[SBUF_MAX];	// USART Send buffer (ring buffer)
volatile uint8_t swpnt;					// Send buffer write pointer
volatile uint8_t srpnt;					// Send biffer read  pointer

//
// Initialize of UART
void UartInitialize(unsigned int baud)
{
	rwpnt = rrpnt = 0;
	swpnt = srpnt = 0;

										// UBRRݒ肷ƂɊۂߏĂB
	unsigned int ubrr = (F_CPU>>4)/baud-1;

	UBRRH = (uint8_t)(ubrr>>8);			// {[[g
	UBRRL = (uint8_t)ubrr;

	UCSRA = 0b00000000;					// M,M荞݋
	UCSRB = (1 << RXEN) | (1 << TXEN) | (1 << RXCIE) | (1 << TXCIE);
	UCSRC = 3 << UCSZ0;					// stopbit 1bit, 8bitM, Nonparity
}
//
// Check of Recive data
int UartIsReceived()
{
    return  (rwpnt !=  rrpnt) ? 1 : 0;
}
// Get of recive data
unsigned char UartGetReceivedData()
{
	unsigned char data;
	
    if ( UartIsReceived() ) {
  	 	data = rbuf[rrpnt++];
    	if (rrpnt >= RBUF_MAX) rrpnt = 0;
	}
	else data = (unsigned char)0;

    return data;
}
//
// Interrput of recive data
ISR(USART_RX_vect)
{
	uint8_t pnt;

	pnt = rwpnt;
    rbuf[rwpnt++] = UDR;						// Set of recive data
    if (rwpnt >= RBUF_MAX) rwpnt = 0;
	if (rwpnt != rwpnt)    rwpnt = pnt;
}
//
// Internal module Send data
static void UartPrivateSendChar()
{
	if (swpnt != srpnt) UDR = sbuf[srpnt++];	// Send data
	if (srpnt >= SBUF_MAX) srpnt = 0;
}
//
// 荞݂ɂ鑗M
ISR(USART_TX_vect)
{
	UartPrivateSendChar();
}
//
// 1oCgM
void UartSendChar(unsigned char c)
{
	uint8_t pnt;

	// Full of send buffer
	pnt = swpnt;
	pnt += 1;
	if (pnt >= SBUF_MAX)  pnt = 0;
	while(pnt == srpnt);

	// Set of send buffer
	sbuf[swpnt] = c;
	swpnt = pnt;

	// Chek of send
	if (UCSRA & (1<<UDRE)) UartPrivateSendChar();
}
//
// Send of string
void UartSendString(unsigned char *p)
{
	while(*p) UartSendChar(*p++);
}

